#!/usr/bin/perl
use strict;
use warnings;

my %SUGAR_FOR = (
    l => 'lex1', f => 'form1', l0 => 'lex0', f0 => 'form0', lf => 'lf1',
    lex => 'lex1', 'form' => 'form1', gram => 'gram1', phrase => 'phrase1', extra => 'extra1', miss => 'miss1',
    punct => 'punct0', case => 'case0', tok => 'tok0', order => 'order0'
);
my %IS_TYPE = ( lex => 1, form => 1, gram => 1, phrase => 1, extra => 1, miss => 1, punct => 1, case => 1, order => 1 );
my %IS_SUBTYPE_FOR = (
    lex => { asp => 1, se => 1, neT => 1, neU => 1, neX => 1, com => 1, unk => 1 },
    form  => { ze     => 1 },
    gram  => { gender => 1, person => 1, number => 1, tense => 1, mod => 1, deg => 1, neg => 1, svuj => 1, numberU => 1 },
    punct => { brack  => 1 },
);
my %IS_SOURCE = ( tok => 1, tagger => 1, lem => 1, parser => 1, tecto => 1, x => 1, trans => 1, syn => 1, '?' => 1 );
my %IS_CIRC = ( 'ne' => 1, num => 1, coord => 1 );

sub get_type_ser_subtype_source_circs {
    my ($marker) = @_;
    my ( $type_ser, @other ) = split /-/, $marker;
    $type_ser = $SUGAR_FOR{$type_ser} || $type_ser;
    my ( $type, $ser ) = $type_ser =~ m/(.*)(.)/;
    die "Unknown type $type in $marker" if $type ne 'lf' && !$IS_TYPE{$type};
    die "Unknown seriousness $ser in $marker" if !defined $ser || $ser !~ /^[01]$/;
    my ( $subtype, $source, %circs );

    for (@other) {
        if    ( $IS_SUBTYPE_FOR{$type}{$_} ) { warn "Two subtypes: $subtype $_ in $marker\n"   if $subtype;   $subtype   = $_; }
        elsif ( $IS_SOURCE{$_} )             { warn "Two sources: $source $_ in $marker\n"     if $source;    $source    = $_; }
        elsif ( $IS_CIRC{$_} )               { warn "Same circumstance twice: $_ in $marker\n" if $circs{$_}; $circs{$_} = 1; }
        else                                 { warn "Unknown marker $_ in $marker (type=$type, ser=$ser)\n"; }
    }
    $subtype ||= 'other';
    $source  ||= 'trans';
    if ( $subtype =~ /ne[TUX]/ ) { $circs{'ne'} = 1; }
    return $type, $ser, $subtype, $source, keys %circs;
}

my ( @s_source, @s_type, @s_source_type, @s_source_circ, @s_type_subtype, @s_circ, @s_type_circ, @s );

sub add_marker {
    my ( $type, $ser, $subtype, $source, @circs ) = @_;
    $s[$ser]++;
    $s_type[$ser]{$type}++;
    $s_source[$ser]{$source}++;
    $s_source_type[$ser]{$source}{$type}++;
    $s_type_subtype[$ser]{$type}{$subtype}++;
    for (@circs) {
        $s_circ[$ser]{$_}++;
        $s_source_circ[$ser]{$source}{$_}++;
        $s_type_circ[$ser]{$type}{$_}++;
    }
    if (!@circs) {
        $s_circ[$ser]{none}++;
        $s_source_circ[$ser]{$source}{none}++;
        $s_type_circ[$ser]{$type}{none}++;
    }
}

while (<>) {
    for (split) {
        next if !s/::[^:]*$//;
        for ( split /::/ ) {
            my ( $type, @other ) = get_type_ser_subtype_source_circs($_);
            if ( $type eq 'lf' ) {
                add_marker( 'lex',  @other );
                add_marker( 'form', @other );
            }
            else { add_marker( $type, @other ); }
        }
    }
}
# 1 percent of serious, minor and both
my $pm = 100 / $s[0];
my $ps = 100 / $s[1];
my $pb = 100 / ($s[0]+$s[1]);

#------------------- Printing
my @ordered_sources = sort { $s_source[1]{$b} <=> $s_source[1]{$a} } keys %IS_SOURCE;
my @ordered_types   = sort { $s_type[1]{$b} <=> $s_type[1]{$a} } keys %IS_TYPE;
my @ordered_circs   = sort { $s_circ[1]{$b} <=> $s_circ[1]{$a} } keys %IS_CIRC;

sub print_sources {
    my ($fn) = @_;
    open my $TEX, '>', $fn if $fn;

    print "\n== Sources ==\n";
    printf "%-8s %6s %6s %6s\n", qw(Source serious minor both);
    for (@ordered_sources) {
        my ( $s, $m ) = ( $s_source[1]{$_} || 0, $s_source[0]{$_} || 0 );
        printf      "%-8s %6s %6s %6s\n", $_, $s, $m, $s + $m;
        printf $TEX "\\Source{%-8s} & %4s & %4.1f\\%% & %4s & %4.1f\\%% & %4s & %4.1f\\%% \\\\\n", $_, $s, $s*$ps, $m, $m*$pm, $s+$m, ($s+$m)*$pb if $fn;
    }
    printf "%-8s %6s %6s %6s\n", 'total', $s[1], $s[0], $s[0]+$s[1];
    print $TEX "\\hline\n\\bf total & \\bf $s[1] & 100\\% & \\bf $s[0] & 100\\% & \\bf ", $s[0]+$s[1] ," & 100\\% \\\\\n" if $fn;
    close $TEX if $fn;
}

sub print_types {
    my ($fn) = @_;
    open my $TEX, '>', $fn if $fn;

    print "\n== Types ==\n";
    printf "%-8s %6s %6s %6s\n", qw(Type serious minor both);
    for (@ordered_types) {
        my ( $s, $m ) = ( $s_type[1]{$_} || 0, $s_type[0]{$_} || 0 );
        printf      "%-8s %6s %6s %6s\n", $_, $s, $m, $s + $m;
        printf $TEX "\\Type{%-8s} & %4s & %4.1f\\%% & %4s & %4.1f\\%% & %4s & %4.1f\\%% \\\\\n", $_, $s, $s*$ps, $m, $m*$pm, $s+$m, ($s+$m)*$pb if $fn;
    }
    printf "%-8s %6s %6s %6s\n", 'total', $s[1], $s[0], $s[0]+$s[1];
    print $TEX "\\hline\n\\bf total & \\bf $s[1] & 100\\% & \\bf $s[0] & 100\\% & \\bf ", $s[0]+$s[1] ," & 100\\% \\\\\n" if $fn;
    close $TEX if $fn;
}

sub print_circs {
    my ($fn) = @_;
    open my $TEX, '>', $fn if $fn;
    my ( $t_s, $t_m ) = ( 0, 0 );

    print "\n== Circumstances ==\n";
    printf "%-8s %6s %6s %6s\n", qw(Circ serious minor both);
    for (@ordered_circs, 'none') {
        my ( $s, $m ) = ( $s_circ[1]{$_} || 0, $s_circ[0]{$_} || 0 );
        printf      "%-8s %6s %6s %6s\n", $_, $s, $m, $s + $m;
        printf $TEX "\\Circs{%-8s} & %4s & %4.1f\\%% & %4s & %4.1f\\%% & %4s & %4.1f\\%% \\\\\n", $_, $s, $s*$ps, $m, $m*$pm, $s+$m, ($s+$m)*$pb if $fn;
        $t_s += $s;
        $t_m += $m;
    }
    printf "%-8s %6s %6s %6s\n", 'total', $t_s, $t_m, $t_s + $t_m;
    print $TEX "\\hline\n\\bf total & \\bf $t_s &  & \\bf $t_m &  & \\bf ", $t_s+$t_m ," &  \\\\\n" if $fn;
    close $TEX if $fn;
}

sub print_subtypes {
    my ($fn) = @_;
    open my $TEX, '>', $fn if $fn;

    print "\n== Subtypes of errors ==\n";
    printf "%-6s %8s %8s %6s %6s\n", qw(Type Subtype serious minor both);
    for my $a_type (@ordered_types) {
        next if !$IS_SUBTYPE_FOR{$a_type} || !%{ $IS_SUBTYPE_FOR{$a_type} };
        print $a_type, "\n";
        print $TEX "\\Type{$a_type} & & & & & & &\\\\\n" if $fn;
        my @subtypes = sort { $s_type_subtype[1]{$a_type}{$b} <=> $s_type_subtype[1]{$a_type}{$a} } keys %{ $s_type_subtype[1]{$a_type} };
        for my $a_subtype (@subtypes) {
            my ($s, $m) = ($s_type_subtype[1]{$a_type}{$a_subtype} || 0, $s_type_subtype[0]{$a_type}{$a_subtype} || 0);
            printf      "%-6s %8s %8s %6s %6s\n", '', $a_subtype, $s, $m, $s + $m;
            printf $TEX " &\\Subtype{%-8s} & %4s & %4.1f\\%% & %4s & %4.1f\\%% & %4s & %4.1f\\%% \\\\\n", $a_subtype, $s, $s*$ps, $m, $m*$pm, $s+$m, ($s+$m)*$pb if $fn;
        }
        print $TEX "\\hline\n" if $fn;
    }
    close $TEX if $fn;
}


sub print_s_sources_types {
    my ($fn) = @_;
    open my $TEX, '>', $fn if $fn;

    print "\n== Sources and types of serious errors ==\n";
    print join( "\t", '', @ordered_types ), "\n";
    print $TEX join( ' & ', '', map {"\\bf $_"} @ordered_types ), " \\\\\n\\hline \\hline\n" if $fn;

    for my $a_source (@ordered_sources) {
        print $a_source;
        print $TEX "\\bf $a_source";
        for my $a_type (@ordered_types) {
            my $value = $s_source_type[1]{$a_source}{$a_type};
            print "\t",  $value || '.';
            print $TEX " & ", $value || ' ' if $fn;
        }
        print "\n";
        print $TEX " \\\\ \\hline\n" if $fn;
    }
    close $TEX if $fn;
}


sub print_s_sources_circs {
    my ($fn) = @_;
    open my $TEX, '>', $fn if $fn;

    print "\n== Sources and circumstances of serious errors ==\n";
    print join( "\t", '', @ordered_circs, 'none' ), "\n";
    print $TEX join( ' & ', '', map {"\\bf $_"} @ordered_circs, 'none' ), " \\\\\n\\hline \\hline\n" if $fn;
    for my $a_source (@ordered_sources) {
        print $a_source;
        print $TEX "\\bf $a_source";
        for my $a_circ (@ordered_circs, 'none') {
            my $value = $s_source_circ[1]{$a_source}{$a_circ};
            print "\t",  $value || '.';
            print $TEX " & ", $value || ' ' if $fn;
        }
        print "\n";
        print $TEX " \\\\ \\hline\n" if $fn;
    }
    close $TEX if $fn;
}

sub print_s_types_circs {
    my ($fn) = @_;
    open my $TEX, '>', $fn if $fn;

    print "\n== Types and circumstances of serious errors ==\n";
    print join( "\t", 'type', @ordered_circs, 'none' ), "\n";
    print $TEX join( ' & ', '', map {"\\bf $_"} @ordered_circs, 'none' ), " \\\\\n\\hline \\hline\n" if $fn;
    for my $a_type (@ordered_types) {
        print $a_type;
        print $TEX "\\bf $a_type";
        for my $a_circ (@ordered_circs, 'none') {
            my $value = $s_type_circ[1]{$a_type}{$a_circ};
            print "\t",  $value || '.';
            print $TEX " & ", $value || ' ' if $fn;
        }
        print "\n";
        print $TEX " \\\\ \\hline\n" if $fn;
    }
    close $TEX if $fn;
}

print_sources('tables/sources.tex');
print_types('tables/types.tex');
print_circs('tables/circs.tex');
print_subtypes('tables/subtypes.tex');
print_s_sources_types('tables/s_sources_types.tex');
print_s_sources_circs('tables/s_sources_circs.tex');
print_s_types_circs('tables/s_types_circs.tex');
